#include <stdio.h>
#include <string.h>

// RC server
#include <arpa/inet.h>        /*  inet (3) funtions         */
#include <errno.h>

/*  Global constants  */
#define SERVER_PORT          (2345)

ssize_t Readline(int sockd, void *vptr, size_t maxlen);

void remote_thread( void *v )
{
	FILE *hlog;

	hlog = fopen( "/mtd_ram/remote.log", "a+" );

	fprintf( hlog, "remote_thread started: pInstance - 0x%08X\n", v );
	fflush(hlog);
	
	int       list_s;
	int       conn_s;
	short int port;  
	struct    sockaddr_in servaddr;
	char      buffer[1024]; 
	char     *endptr;    

	if ( (list_s = socket(AF_INET, SOCK_STREAM, 0)) < 0 )
	{
		fprintf(hlog, "Error creating listening socket.\n");
		fclose(hlog);
		return;
    	}

	int on = 1;
	setsockopt(list_s, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(int));

	memset(&servaddr, 0, sizeof(servaddr));
	servaddr.sin_family      = AF_INET;
	servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servaddr.sin_port        = htons(SERVER_PORT);

	if ( bind(list_s, (struct sockaddr *) &servaddr, sizeof(servaddr)) < 0 ) 
	{
		fprintf(hlog, "Error calling bind()\n");
		fclose(hlog);
		return;
	}

	if ( listen(list_s, 10) < 0 ) 
	{
		fprintf(hlog, "Error calling listen()\n");
		fclose(hlog);
		return;
	}

	while (1)
	{
		if ( (conn_s = accept(list_s, NULL, NULL) ) < 0 ) {
		    fprintf( hlog, "ECHOSERV: Error calling accept()\n");
		    break;
		}

		memset( buffer, 0, 1024 );
		Readline(conn_s, buffer, 1023);

		if ( buffer[0] == 'Q' )
		{
		    fprintf( hlog, "exit..\n");
			close(conn_s);
			break;
		}

		int key = atoi( buffer );

		fprintf( hlog, "received key: %d\n", key );
		fflush(hlog);

		asm volatile(
			"MOV	R0,%0\n"
			"MOV 	R1,%1\n"
			"BL	_ZN14SsKeyInputBase7SendKeyEi\n"
			:
			: "r" (v), "r" (key) );

		if ( close(conn_s) < 0 ) 
		{
		    fprintf(hlog, "Error calling close()\n");
		    break;
		}
	}

	fclose(hlog);
	close(list_s);
	return;
}

ssize_t Readline(int sockd, void *vptr, size_t maxlen) {
    ssize_t n, rc;
    char    c, *buffer;

    buffer = vptr;

    for ( n = 1; n < maxlen; n++ ) {
	
	if ( (rc = read(sockd, &c, 1)) == 1 ) {
	    *buffer++ = c;
	    if ( c == '\n' )
		break;
	}
	else if ( rc == 0 ) {
	    if ( n == 1 )
		return 0;
	    else
		break;
	}
	else {
	    if ( errno == EINTR )
		continue;
	    return -1;
	}
    }

    *buffer = 0;
    return n;
}

